'use strict';

module.exports = function(indicatorMixin, accessor_ohlc, indicator_atr) {  // Injected dependencies
  return function() { // Closure function
    var p = {},  // Container for private, direct access mixed in variables
        multiplier = 3,
        atr = indicator_atr();

    function indicator(data) {
      atr.accessor(p.accessor).period(p.period).init();

      return data.map(function(d, i) {
        var close = p.accessor.c(d),
            stop = atr.atr(d)*multiplier;
        if(i >= p.period) return { date: p.accessor.d(d), close: close, up: close-stop, down: close+stop };
        else return { date: p.accessor.d(d), up: null, down: null };
      })
      .filter(function(d) { return d.up !== null && d.down !== null; }) // Filter out empties
      .reduce(function(result, d, i) { // Reduce to access the previous result array
        var prev = result[i-1],
            up = i === 0 ? d.up : null, // Always start with an up trend?
            down = null;

        if(prev && prev.up !== null) {
          if(d.close > prev.up) up = Math.max(d.up, prev.up);
          else down = d.down;
        }

        if(prev && prev.down !== null) {
          if(d.close < prev.down) down = Math.min(d.down, prev.down);
          else up = d.up;
        }

        result.push({ date: d.date, up: up, down: down });
        return result;
      }, []);
    }

    indicator.multiplier = function(_) {
      if (!arguments.length) return multiplier;
      multiplier = _;
      return indicator;
    };

    // Mixin 'superclass' methods and variables
    indicatorMixin(indicator, p)
      .accessor(accessor_ohlc())
      .period(14);

    return indicator;
  };
};